import matplotlib.pyplot as plt
import numpy as np

def draw_som(case_labels, xi, m):
    JM = len(xi)
    n = len(xi[0])
    g_size = len(m)

    #
    #       参照ベクトルの傾き
    #
    slopes = np.empty((g_size, g_size))
    for i in range(0, g_size):
        for j in range(0, g_size):
            count = 0
            v = 0.0
            for ix in range(-1, 2):
                for jy in range(-1, 2):
                    try:
                        if ix != 0 or jy != 0:
                            if (i+ix) >= 0 and (j+jy) >= 0:
                                v += np.absolute(m[i+ix][j+jy] - m[i][j]).sum()
                                count += 1
                    except:
                        pass
           
            slopes[i][j] = v / count 
    slope_max = np.ma.max(slopes[0:g_size, 0:g_size])
    slope_min = np.ma.min(slopes[0:g_size, 0:g_size])

    #       日本語表記の設定
    plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP',
                                   'Hiragino Maru Gothic Pro']

    #
    #       傾きに応じた色付け
    #
    plt.figure(figsize = (7, 7))
    plt.axis('off')
    plt.title('自己組織化マップ\nSOM', fontsize = 20)
    for i in range(0, g_size):
        for j in range(0, g_size):
            vc = (slopes[i][j] - slope_min) / (slope_max - slope_min)
            plt.fill_between([i-0.5, i+0.5], [j-0.5, j-0.5], [j+0.5, j+0.5],
                             color = (vc, 1.0-vc, 0.0), alpha = 0.5)

    m_c = []
    diffs = np.empty((g_size, g_size))
    plt.xticks([-1, g_size+1], [' ', ' '])
    plt.yticks([-1, g_size+1], [' ', ' '])
    for IN in range(JM):
        #
        #       勝者ベクトル
        #
        for i in range(g_size):
            for j in range(g_size):
                diffs[i][j] = ((m[i][j] - xi[IN]) ** 2).sum()
        tc = np.argmin(diffs)
        tc_i = tc // g_size
        tc_j = tc % g_size
        #
        #       勝者ベクトルの位置にプロット
        #
        plt.text(tc_i, tc_j, case_labels[IN])

    plt.show()


